function ret = alpha_exp(m, n, k, lambda_list, pos_ratio, rho_list, alpha_range)
if nargin == 0
	disp('ret = alpha_exp(m, n, k=10, lambda=[1:12], pos_ratio=0.2, rho_list=[0.01 0.05 0.10 0.95], alpha_range=2.^-[-20:2:8])');
end
if nargin <= 2
	k = 10;
end
if nargin <= 3
	lambda_list = [1:12];
end
if nargin <= 4
	pos_ratio = 0.2;
end
if nargin <= 5
	rho_list = [0.01 0.05 0.10 0.95];
end
if nargin <= 6
	alpha_range = 2.^[-20:2:8];
end

oc = synthetic_gen(m, n, k, pos_ratio, rho_list);

final = struct();
for l = 1:length(lambda_list)
	lambda = 2^-lambda_list(l);
	ret = struct();
	for r = 1:length(rho_list)
		map = zeros(1,0);
		auc = zeros(1,0);
		hlu = zeros(1,0);
		ndcg = zeros(10,0);
		for i = 1:length(alpha_range)
			alpha = alpha_range(i);
			fprintf(1,'running lambda %g rho %g alpha %g\n', lambda, alpha, rho_list(r))
			m = pmf_train(oc.R{r},[],sprintf('-s 11 -k %d -t 20 -b 0 -r %g -l %g -n 8', k, alpha, lambda));
			res = pmf_predict_ranking(oc.Rt{r},m.W,m.H,10,[]);
			t = pmf_eval_pos_rank(res.pos_rank, 10);
			res.hlu = t.hlu;

			hlu = [hlu res.hlu];
			map = [map res.map];
			auc = [auc res.auc];
			ndcg = [ndcg reshape(res.ndcg, 10,1)];
		end
		ret.map{r} = map;
		ret.auc{r} = auc;
		ret.hlu{r} = hlu;
		ret.ndcg{r} = ndcg;
	end
	if l == 1,
		final = ret;
	else
		for r = 1:length(rho_list)
			final.map{r} = max(ret.map{r}, final.map{r});
			final.auc{r} = max(ret.auc{r}, final.auc{r});
			final.hlu{r} = max(ret.hlu{r}, final.hlu{r});
			final.ndcg{r} = max(ret.ndcg{r}, final.ndcg{r});
		end
	end
	%final = setfield(final, sprintf('l%d',l), ret);
end
final.range = alpha_range;
final.oc = oc;
ret = final; 
end

function oc = synthetic_gen(m, n, k, pos_ratio, rho_list)
if nargin < 5
	disp('oc = synthetic_gen(m, n, k, pos_ratio, rho_list)');
	return;
end
oc.dim = [m n];
W = randn(m,k);
H = randn(n,k);
A = W*H';
v = sort(A(:),'descend');
threshold = v(ceil(m*n*pos_ratio));
A(A < threshold) = 0;
A(A >= threshold) = 1;
[I J V] = find(sparse(A));
all_pos = [I J V];

for i = 1:length(rho_list)
	rho = rho_list(i);
	r = rand(size(all_pos,1),1);
	tmp.R = all_pos(r < rho, :);
	tmp.Rt = all_pos;
	%tmp.Rt = all_pos(r >= rho, :);
	%tmp.Rt = all_pos(r >= 0.95, :);
	tmp.R(:,3) = 1;
	tmp.Rt(:,3) = 1;
	oc.R{i} = tmp.R;
	oc.Rt{i} = tmp.Rt;
end
oc.W = W;
oc.H = H;
oc.pos_ratio = pos_ratio;
oc.rho_list = rho_list;
end
